from pymel.all import *
import maya.cmds as cmds

''' METHODS '''

def reference_planes():

    # Make anim used to move reference planes
    refAnim = curve(p=[(-.5,0,0),(.5,0,0)], degree=1, name="reference_handle")
    for a in ['tx', 'rx', 'ry', 'rz', 'sz']:
        setAttr(refAnim.attr(a), l=1, k=0, cb=0)
        
    # Double the reference anims for the front and the side
    refAnimFrontGrp = group(refAnim, name="reference_handle_front_grp")
    refAnimSideGrp = instance(refAnimFrontGrp, name="reference_handle_side_grp")[0]

    # Orient the anims so positive z movement is outwards
    refAnimFrontGrp.ry.set(180)
    refAnimSideGrp.ry.set(-90)

    # Create and constrain planes
    frontPlane = polyPlane(w=1, h=1, sw=1, sh=1, n="front_ref_geo")[0]
    frontPlane.tz.set(-.5)
    xform(frontPlane, rp=[0,0,0], ws=1)
    xform(frontPlane, sp=[0,0,0], ws=1)
    frontPlane.rx.set(90)
    makeIdentity(frontPlane, t=1, r=1, s=1, a=1)

    sidePlane = duplicate(frontPlane, n="side_ref_geo")[0]

    parentConstraint(refAnim, frontPlane, mo=0)
    scaleConstraint(refAnim, frontPlane, mo=0)

    otherRefAnim = refAnimSideGrp.getChildren()[0]
    parentConstraint(otherRefAnim, sidePlane, mo=0)
    scaleConstraint(otherRefAnim, sidePlane, mo=0)

    for p in [frontPlane, sidePlane]:
        for a in ['tx', 'ty', 'tz', 'rx', 'ry', 'rz', 'sx', 'sy', 'sz']:
            setAttr(p.attr(a), l=1, k=0, cb=0)

    # Set default plane position
    refAnim.tz.set(1)

    '''
    # Set up nodeless scaling system to double the current translateZ value
    systemGrp = group(em=1, n="scalar_grp")
    origLoc = spaceLocator(n="orig_value")
    scaleLoc = spaceLocator(n="scaled_value")
    scalingGrp = group(em=1, n="scaling")
    parent(origLoc, scalingGrp)
    parent(scaleLoc, systemGrp)
    scalingGrp.scale.set(2,2,2)
    parent(scalingGrp, systemGrp, r=1)
    parentConstraint(origLoc, scaleLoc)
    hide(systemGrp)

    refAnim.tz >> origLoc.tz

    # Limit scale to extent of the plane moved out
    refAnim.maxScaleLimitEnable.set(1,1,0)
    scaleLoc.tz >> refAnim.maxScaleXLimit
    scaleLoc.tz >> refAnim.maxScaleYLimit
    '''

    # Organize under root group
    rootGrp = group([refAnimFrontGrp, refAnimSideGrp, frontPlane, sidePlane], n="reference_planes_grp")

    # Create display layer for planes
    if objExists('reference_lyr'):
        editDisplayLayerMembers('reference_lyr', [frontPlane, sidePlane])
    else:
        createDisplayLayer([frontPlane, sidePlane], n='reference_lyr')
        
    # Add slider system
    faderRoot = transparency_fader( [frontPlane, sidePlane] )
    faderAnim = selected()[0]
    select(d=1)
    parent(faderRoot, rootGrp)
    
    # Adjust slider default location and constrain it to the image plane handles
    faderRoot.inheritsTransform.set(0)
    faderAnim.ControlMoverVisibility.set(0)
    faderRoot.ty.set(.333*faderRoot.ty.get())
    
    frontPlaneLoc = spaceLocator(n="front_plane_loc")
    hide(frontPlaneLoc)
    parent(frontPlaneLoc, rootGrp)
    pointConstraint(refAnim, frontPlaneLoc)
    
    sidePlaneLoc = spaceLocator(n="side_plane_loc")
    sidePlaneLocGrp = group(em=1, n="side_plane_loc_grp")
    sidePlaneLocGrp.ry.set(180)
    parent(sidePlaneLoc, sidePlaneLocGrp)
    hide(sidePlaneLocGrp)
    parent(sidePlaneLocGrp, rootGrp)
    pointConstraint(otherRefAnim, sidePlaneLoc)

    sidePlaneLoc.tx >> faderRoot.tx
    frontPlaneLoc.tz >> faderRoot.tz
    
    refAnim.sx >> faderRoot.sx
    refAnim.sy >> faderRoot.sy
    refAnim.sx >> faderRoot.sz
    
    
    

def transparency_fader( polyObjs, useConstraint=False ):
    
    ## CURVE CREATION AND MODIFICATION METHODS ##
    
    def createCurve_DownArrow(name="downArrowCurve"):
        c = curve(n=name, d=1, p= ( [0.5, 0, 0], [0.5, -1, 0], [1, -1, 0], [0, -2, 0],\
                                    [-1, -1, 0], [-0.5, -1, 0], [-0.5, 0, 0], [0.5, 0, 0]))
        return c
    
    def twistShapeAndCombine(obj):
        objDup = duplicate(obj)[0]
        objDup.ry.set(90)
        makeIdentity(objDup, a=1)
        objDupShape = objDup.getShape()
        parent(objDupShape, obj, s=1, add=1)
        delete(objDup)
        return obj
        
    
    ## SETUP SLIDER BASED ON FIRST SELECTED POLYGON OBJECT ##
    
    polyObj = polyObjs[0]
    
    # Get bounds of polygon object, use to determine fader position and scale    
    bounds = polyEvaluate(polyObj, b=1)
    distance = math.sqrt((bounds[0][0] - bounds[0][1])**2 + (bounds[1][0] - bounds[1][1])**2)
    FADER_SIZE = distance / 3.0
    FADER_SCALE = FADER_SIZE / 7.0
    faderPos = [bounds[0][1] + (FADER_SIZE / 3.0), bounds[1][1] + (FADER_SIZE / 3.0), bounds[2][1]]
    
    # Create fader control
    ctrlFader = circle(n='transparency_fader')[0]
    
    twistShapeAndCombine(ctrlFader)
    
    ctrlFader.translate.set(faderPos)
    ctrlFader.scale.set(FADER_SCALE, FADER_SCALE, FADER_SCALE)
    makeIdentity(ctrlFader, a=1, t=0, r=1, s=1)
    
    # Create fade slider movement control
    ctrlMover = twistShapeAndCombine(createCurve_DownArrow('transparency_fader_mover'))
    
    ctrlMover.translate.set(faderPos)
    ctrlMover.scale.set(FADER_SCALE, FADER_SCALE, FADER_SCALE)
    makeIdentity(ctrlMover, a=1, t=0, r=1, s=1)
    
    # Create fader curve
    faderCurve = curve(n='transparency_fader_curve', d=1, p=[(faderPos[0], faderPos[1], faderPos[2]),\
     (faderPos[0], faderPos[1] + FADER_SIZE, faderPos[2])], k=[0,1])
    faderCurve.attr('overrideEnabled').set(1)
    faderCurve.attr('overrideDisplayType').set(2)
    
    # Create fader top groups
    transpGroup = group(n='transparency_slider', em=1)
    transpGroup.translate.set(faderPos)
    faderCtrlGroup = group(n='fader_control_group', em=1)
    faderCtrlGroup.scale.set(FADER_SIZE, FADER_SIZE, FADER_SIZE)
    faderCtrlGroup.translate.set(faderPos)
    
    # Arrange the parent hierarchy
    parent(faderCurve, faderCtrlGroup)
    parent(ctrlFader, faderCtrlGroup)
    parent(faderCtrlGroup, transpGroup)
    parent(ctrlMover, transpGroup)
    parentConstraint(ctrlMover, faderCtrlGroup, mo=1)
    
    # Lock uneeded attributes and set translate limits on the faders controls
    transforms = ['tx', 'ty', 'tz', 'rx', 'ry', 'rz', 'sx', 'sy', 'sz']
    
    for trans in transforms:
        ctrlFader.attr(trans).set(lock=1, keyable=0, cb=0)
        faderCurve.attr(trans).set(lock=1, keyable=0, cb=0)
    
    ctrlFader.ty.set(lock=0, keyable=0, cb=1)
    ctrlFader.visibility.set(lock=1, keyable=0, cb=0)
    
    transformLimits(ctrlFader, ety=(1, 1), ty=(0, 1))
    
    # Lock uneeded attributes on the fader mover
    for trans in ['sx', 'sy', 'sz']:
        ctrlMover.attr(trans).set(lock=1, keyable=0, cb=0)
    
    for trans in ['tx', 'ty', 'tz', 'rx', 'ry', 'rz']:
        ctrlMover.attr(trans).set(lock=0, keyable=0, cb=1)
        
    # Lock uneeded attributes on the fader control group
    for trans in transforms+["visibility"]:
        faderCtrlGroup.attr(trans).set(lock=1, keyable=0, cb=0)
        
    # Connect visibility of control mover to a custom attribute on the fader control
    ctrlFader.addAttr("ControlMoverVisibility", at="bool", dv=1)
    ctrlFader.ControlMoverVisibility.connect(ctrlMover.visibility)
    ctrlFader.ControlMoverVisibility.set(lock=0, keyable=0, cb=1)
    ctrlMover.visibility.set(lock=1, keyable=0, cb=0)
    
    ## useConstraint FLAG ##
    # Constrain fader to polygon object
    if (useConstraint): parentConstraint(polyObj, transpGroup, mo=1)
    
    # Set default control position to half
    ctrlFader.ty.set(.5)
    
    
    ## CONNECT SLIDER TO SHADERS ##
    
    for polyObj in polyObjs:
        
        # Get polygon's shader
        shEng = polyObj.listRelatives(c=1, s=1)[0].listConnections(t='shadingEngine')[0]
        shader = listConnections(shEng.name() + ".surfaceShader")
        
        # Creates a new shader if there is none or is using lambert1
        if not shader or 'lambert1' == shader[0]:
            shader = shadingNode('lambert', asShader=1, n = polyObj.name() + '_transparency_slider_shader');
            select(cl=1)
            select(polyObj)
            hyperShade(assign = shader)
        else:
        	shader = shader[0]
        
        shader.transparency.disconnect()
        
        # Fader control's translate y drives transparency of shader
        for colorAttr in [shader.transparencyR, shader.transparencyG, shader.transparencyB]:
            setDrivenKeyframe(colorAttr, cd=ctrlFader.ty, dv=1, v=0, itt="linear", ott="linear")
            setDrivenKeyframe(colorAttr, cd=ctrlFader.ty, dv=0, v=1, itt="linear", ott="linear")
            
    # Select the fader
    select(ctrlFader)
    return transpGroup

''' COMMANDS '''
    
def transparency_fader_cmd():
    
    polyObjs = filter(lambda obj: 'mesh' == nodeType(obj.listRelatives(c=1, s=1)[0]), ls(sl=1))
    
    if len(polyObjs) > 0:
        transparency_fader(polyObjs)
    else:
        raise Exception('Select at least one polygon object.')
        
def reference_planes_cmd():

    reference_planes()